def test_app_save_note():
    pass


def test_app_update_note_title():
    pass


def test_app_update_note_body():
    pass


if __name__ == "__main__":
    test_app_save_note()
    test_app_update_note_title()
    test_app_update_note_body()
